--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

-- $Revision: 11 $
-- $Date: 2008-12-11 20:50:55 +0000 (Thu, 11 Dec 2008) $
if( not Broker_GarbageFu ) then
    return;
end--if
------------------------------------------------------------------------------------------------------
-- Auction value functions
------------------------------------------------------------------------------------------------------

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Broker_GarbageFu" );
local dewdrop = LibStub( "Dewdrop-2.0" );
local PT3 = LibStub( "LibPeriodicTable-3.1" );

Broker_GarbageFu.vars.auctionaddons = {
    ["-CA-"] = { name=L["Cached Auctioneer"], func="GetCachedAuctioneerAuctionValue", check="GarbageFu" },
    ["auctioneer_advanced"] = { name="AucAdvanced", func="GetAuctioneerAdvancedAuctionValue", check="AucAdvanced" },
    ["auctioneer"] = { name="Auctioneer", func="GetAuctioneerAuctionValue", check="Auctioneer" },
    ["kc_items"] = { name="KC_Items", func="GetKCItemsAuctionValue", check="KC_Auction" },
    ["wowecon"] = { name="WoWEcon", func="GetWoWEconAuctionValue", check="WOWEcon_Enabled" },
    ["-DNU-"] = { name=L["-- Don't use the addons below --"], func="", check="GarbageFu", stop=true },
};
Broker_GarbageFu.vars.auctionaddonsorder = {
    "-CA-",
    "auctioneer_advanced",
    "auctioneer",
    "kc_items",
    "wowecon",
    "-DNU-",
};

function Broker_GarbageFu:InitAuction()
    self:InitAuctionAddonOrder();
    self:MakeAuctionSetsTables();
    self:HookAuctioneer();
    self:HookKCItems();
end--Broker_GarbageFu:InitAuction()

function Broker_GarbageFu:GetAuctionThreshold()
    return self.db.profile.auctionThreshold;
end--Broker_GarbageFu:GetAuctionThreshold()

function Broker_GarbageFu:IsAuctionThreshold( value )
    return self.db.profile.auctionThreshold == tonumber( value );
end--Broker_GarbageFu:IsAuctionThreshold( value )

function Broker_GarbageFu:SetAuctionThreshold( value )
    self.db.profile.auctionThreshold = tonumber( value );
    self:UpdateDisplay();
end--Broker_GarbageFu:SetAuctionThreshold( value )

-- Public ask auction price function
function Broker_GarbageFu:GetAuctionValue( item )
    if( self:IsItemAuctionPrice( item ) and ( not self:IsItemSoulbound( item ) ) ) then
        for _, addon in ipairs( self.db.profile.auctionaddonsorder ) do
            if( self.vars.auctionaddons[addon] ) then
                if( self.vars.auctionaddons[addon].stop ) then
                    break;
                end--if
                item.value, item.pricetype = self[self.vars.auctionaddons[addon].func]( self, item );
                if( item.value ) then
                    break;
                end--if
            end--if
        end--for
        -- Store in cache
        if( item.value ) then
            item.totvalue = item.value * item.stack;
            return true;
        end--if
    end--if
    return false;
end--Broker_GarbageFu:GetAuctionValue( item )

--------------------
-- Auctioneer cached
function Broker_GarbageFu:GetCachedAuctioneerAuctionValue( item )
    if( self.db.realm.auctioncache[item.id] ) then
        return self.db.realm.auctioncache[item.id], "|cff00FF00AC|r";
    end--if
end--Broker_GarbageFu:GetCachedAuctioneerAuctionValue( item )

--------------------
-- Auctioneer
function Broker_GarbageFu:GetAuctioneerAuctionValue( item )
    if( ( not Auctioneer ) or ( not Auctioneer.Util ) ) then
        return;
    end--if

    local itemID, randomProp, enchant, uniqID, lame = EnhTooltip.BreakLink( item.link );

    local auctKey = Auctioneer.Util.GetHomeKey();
    local itemKey = itemID .. ":" .. randomProp .. ":" .. enchant;

    local median, medCount = Auctioneer.Statistic.GetUsableMedian( itemKey, auctKey );

    if( median ) then
        return median, "|cff00FF00AA|r";
    end--if
end--Broker_GarbageFu:GetAuctioneerAuctionValue( item )

--------------------
-- Auctioneer Advanced
function Broker_GarbageFu:GetAuctioneerAdvancedAuctionValue( item )
    if( ( not AucAdvanced ) or ( not AucAdvanced.API ) ) then
        return;
    end--if
    
    local algoList = AucAdvanced.API.GetAlgorithms();
    --for now use algorithm 1, will need to let the user decide
    local median = AucAdvanced.API.GetAlgorithmValue( algoList[1], item.link );

    if( median ) then
        return median, "|cff00FF00AA|r";
    end--if
end--Broker_GarbageFu:GetAuctioneerAuctionValue( item )

--------------------
-- KC_Items
function Broker_GarbageFu:GetKCItemsAuctionValue( item )
    if( not KC_Auction ) then
        return;
    end--if

    local _, _, _, _, _, _, vs = KC_Auction:GetItemData( item.code );

    if( vs and ( vs ~= "0" ) and ( vs ~= 0 ) ) then
        return tonumber( vs ), "|cff00FF00AK";
    end--if
end--Broker_GarbageFu:GetKCItemsAuctionValue( item )

--------------------
-- WoWEcon    
function Broker_GarbageFu:GetWoWEconAuctionValue( item )
    if( not WOWEcon_Enabled ) then
        return;
    end--if

    local value, _, _ = WOWEcon_GetAuctionPrice_ByLink( item.link );

    if( value and ( value > 0 ) ) then
        return value, "|cff00FF00AW";
    end--if
end--Broker_GarbageFu:GetWoWEconAuctionValue( item )

-------------------------------------------------------------------------------------
-- Auctioneer cache functions
-------------------------------------------------------------------------------------

function Broker_GarbageFu:ClearAuctioneerCache()
    self.db.realm.auctioncache = {};
    self:UpdateAllItemValues();
end--Broker_GarbageFu:ClearAuctioneerCache()

local cachedItems = 0;

function Broker_GarbageFu:CacheAuctioneer()
    if( not ( AucAdvanced or Auctioneer ) ) then
        self:MyPrint( L["AuctioneerNotLoaded"] );
        return;
    end--if
    cachedItems = 0;
    for i, _ in pairs( self.db.realm.auctioncache ) do  -- First update all prices in the cache already
        self:CacheAuctioneerId( i, true );
    end    
    for i, _ in pairs( self.db.profile.auctionitem ) do  -- Then add any items set to use auction price
        self:CacheAuctioneerId( i );
    end
    self:CacheAuctioneerSet( self.vars.auctionsets ); -- and all items in the auction sets
    self:UpdateAllItemValues();
    self:MyPrint( L["Updated Cached Auctioneer with prices for %s items."], tostring( cachedItems ) );
end--Broker_GarbageFu:CacheAuctioneer()

function Broker_GarbageFu:CacheAuctioneerSet( set )
    for _, s in pairs( set ) do
        for i in PT3:IterateSet( s ) do
            self:CacheAuctioneerId( i );
        end--for
    end--for
end--Broker_GarbageFu:CacheAuctioneerSet( set )

function Broker_GarbageFu:CacheAuctioneerId( id, force )
    if( Auctioneer ) then
        if( ( not force ) and self.db.realm.auctioncache[id] ) then
            return;
        end--if
    
        local auctKey = Auctioneer.Util.GetHomeKey();
        local itemKey = tostring( id ) .. ":0:0";
        local median = Auctioneer.Statistic.GetUsableMedian( itemKey, auctKey );
    
        if( median ) then
            self.db.realm.auctioncache[id] = median;
            cachedItems = cachedItems + 1;
        end--if
    elseif( AucAdvanced ) then
        local algoList = AucAdvanced.API.GetAlgorithms();
        --for now use algorithm 1, will need to let the user decide
        local link = GetItemLink( id );
        local median = AucAdvanced.API.GetAlgorithmValue( algoList[1], link );
    
        if( median ) then
            self.db.realm.auctioncache[id] = median;
            cachedItems = cachedItems + 1;
        end--if
    end--if
end--Broker_GarbageFu:CacheAuctioneerId( id, force )

-------------------------------------------------------------------------------------------
---- Auction hooks functions
-------------------------------------------------------------------------------------------
function Broker_GarbageFu:CheckAuctionAddonLoaded( addon )
    addon = addon:lower();
    if( ( addon == "auctioneer" ) or
             ( addon == "auctioneer_advanced" ) or
             ( addon == "kc_items" ) or
             ( addon == "wowecon" ) ) then
        self:ScheduleEvent( self.DelayedAuctionAddonLoad, 5, self, addon );
    end
end--Broker_GarbageFu:CheckAuctionAddonLoaded( addon )

function Broker_GarbageFu:DelayedAuctionAddonLoad( addon )
    if( ( addon == "auctioneer" ) or ( addon == "auctioneer_advanced" ) ) then
        self:HookAuctioneer();
    elseif( addon == "kc_items" ) then
        self:HookKCItems(); 
    elseif( addon == "wowecon" ) then
        -- WoWEcon don't have a auction house scan
    else
        return;
    end
    if( self:InitAuctionAddonOrder() ) then
        self:UpdateAllItemValues()
    end--if
end--Broker_GarbageFu:DelayedAuctionAddonLoad( addon )
    
function Broker_GarbageFu:HookAuctioneer()
    if( Auctioneer and Auctioneer.Event and ( type( Auctioneer.Event ) == "table" ) ) then
        if( not self:IsHooked( Auctioneer.Event, "StopAuctionScan" ) ) then
            self:Hook( Auctioneer.Event, "StopAuctionScan", "AuctioneerStopScan" );
        end--if
    end--if
    --Need to add for Auctioneer Advanced, it will need to be a module...
end--Broker_GarbageFu:HookAuctioneer()

function Broker_GarbageFu:AuctioneerStopScan()
    self:DebugPrint( "Auctioneer.Event.StopAuctionScan detected!" );
    if( self:IsCachedAuctioneerUsed() ) then
        self:CacheAuctioneer();
    else
        self:UpdateAllItemValues();
    end--if
end--Broker_GarbageFu:AuctioneerStopScan()
        
function Broker_GarbageFu:HookKCItems()
    if( KC_Auction ) then
        if( not self:IsHooked( KC_Auction,"ScanDone" ) ) then
            self:Hook( KC_Auction, "ScanDone", "KCItemsFinishedScan" );
        end--if
        if( not self:IsHooked( KC_Auction,"ScanCanceled" ) ) then
            self:Hook( KC_Auction, "ScanCanceled", "KCItemsStopScan" );
        end--if
    end--if
end--Broker_GarbageFu:HookKCItems()

function Broker_GarbageFu:KCItemsFinishedScan( object )
    self:DebugPrint( "KCItems:ScanDone detected!" );
    self:UpdateAllItemValues();
end--Broker_GarbageFu:KCItemsFinishedScan( object )

function Broker_GarbageFu:KCItemsStopScan( object )
    self:DebugPrint( "KCItems:ScanCanceled detected!" );
    self:UpdateAllItemValues();
end--Broker_GarbageFu:KCItemsStopScan( object )

-------------------------------------------------------------------------------------------
---- Auction sets functions
-------------------------------------------------------------------------------------------

-- Converts self.db.profile.auctionsets into self.vars.auctionsets
-- Needed because self.db.profile.auctionsets is index by name and self.vars.auctionsets needs to be indexed by number with names as string.
function Broker_GarbageFu:MakeAuctionSetsTables()
    self.vars.auctionsets = {};
    for n, _ in pairs( self.db.profile.auctionsets ) do
        table.insert( self.vars.auctionsets, n );
    end--for
end--Broker_GarbageFu:MakeAuctionSetsTables()

function Broker_GarbageFu:IsItemAuctionPrice( item )
    if( self.db.profile.auctionitem[item.id] ) then
        return true;
    end--if
    if( self:IsItemInSet( item.id, self.vars.auctionsets ) ) then
        return true;
    end--if
    if( ( not item.notseen ) and ( item.qual >= self.db.profile.auctionThreshold ) ) then
        return true;
    end--if
end--Broker_GarbageFu:IsItemAuctionPrice( item )

function Broker_GarbageFu:IsItemInAuctionSets( itemid )
    return self:IsItemInSet( itemid, self.vars.auctionsets );
end--Broker_GarbageFu:IsItemInAuctionSets( itemid )

function Broker_GarbageFu:IsAuctionSetSelected( set, subset )
    return self:IsSetSelected( set, subset, self.db.profile.auctionsets );
end--Broker_GarbageFu:IsAuctionSetSelected( set, subset )

function Broker_GarbageFu:IsChildAuctionSetSelected( set )
    return self:IsChildSetSelected( set, self.db.profile.auctionsets );
end--Broker_GarbageFu:IsChildAuctionSetSelected( set )

function Broker_GarbageFu:ToggleAuctionSet( set,subset )
    self:ToggleSets( set, subset, self.vars.sets, self.db.profile.auctionsets );
    self:MakeAuctionSetsTables();
    self:UpdateAllItemValues();
end--Broker_GarbageFu:ToggleAuctionSet( set,subset )

-------------------------------------------------------------------------------------------
---- Drop down functions
-------------------------------------------------------------------------------------------

function Broker_GarbageFu:IsAuctionItem( itemid )
    return self.db.profile.auctionitem[itemid] ~= nil;
end--Broker_GarbageFu:IsAuctionItem( itemid )

function Broker_GarbageFu:ToggleAuctionItem( itemid )
    if( self.db.profile.auctionitem[itemid] ) then
        self.db.profile.auctionitem[itemid] = nil;
    else
        self.db.profile.auctionitem[itemid] = true;
    end--if
    self:UpdateItemValue( itemid );
end--Broker_GarbageFu:ToggleAuctionItem( itemid )

function Broker_GarbageFu:GetAuctionItemTable()
    self.vars.temptbl = {};

    local tbl2 = {};

    -- Start with itemid's listed as auction
    for i, _ in pairs( self.db.profile.auctionitem ) do
        local item = {
            id = i,
        };

        item.name, _, item.qual = GetItemInfo( i );
        if( item.name ) then
            table.insert( self.vars.temptbl, item );
            tbl2[i] = true;
        else
            item = nil;
        end--if
    end--for
    -- Add items in inventory not in keepsets, keeptype or keepitem. Not poor quality or soulbound items.
    for _, i in ipairs( self.vars.items ) do
        if( ( not i.notseen ) and ( i.qual > 0 ) and ( not tbl2[i.id] ) and ( not self:IsItemKeepItem( i.id ) ) and
                 ( not self:IsItemInAuctionSets( i.id ) ) and ( not self:IsItemInKeepSets( i.id ) ) and
                 ( not self:IsItemKeepType( i ) ) and ( not self:IsItemSoulbound( i ) ) and
                 ( i.qual < self.db.profile.auctionThreshold ) ) then
            local item = {
                id = i.id,
                name = i.name,
                qual = i.qual,
            };

            table.insert( self.vars.temptbl, item );
            tbl2[i.id] = true;
        end--if
    end--for
    tbl2 = nil;
    table.sort( self.vars.temptbl, function( a, b ) return a.name < b.name end );
    return self.vars.temptbl;
end--Broker_GarbageFu:GetAuctionItemTable()

function Broker_GarbageFu:ReorderAuctionAddon( index )
    local aname = table.remove( self.db.profile.auctionaddonsorder, index );

    if( index == 1 ) then -- move down
        table.insert( self.db.profile.auctionaddonsorder, index + 1, aname );
    else -- move up
        table.insert(self.db.profile.auctionaddonsorder, index - 1, aname);
    end--if
    self:UpdateAllItemValues();
end--Broker_GarbageFu:ReorderAuctionAddon( index )

function Broker_GarbageFu:IsCachedAuctioneerUsed()
    return next( self.db.realm.auctioncache ) ~= nil;
end--Broker_GarbageFu:IsCachedAuctioneerUsed()
-------------------------------------------------------------------------------------------
-- Auction init functions
-------------------------------------------------------------------------------------------

function Broker_GarbageFu:InitAuctionAddonOrder()
    local changed = false;

    for _, addon in pairs( self.vars.auctionaddonsorder ) do
        local found = false;

        for i, order in ipairs( self.db.profile.auctionaddonsorder ) do
            if( order == addon ) then
                local checkname = _G[self.vars.auctionaddons[addon].check];

                if( not checkname ) then
                    table.remove( self.db.profile.auctionaddonsorder, i ); -- Remove addons not loaded
                    changed = true;
                end--if
                found = true;
                break;
            end--if
        end--for
        if( not found ) then
            local checkname = _G[self.vars.auctionaddons[addon].check];

            if( checkname ) then
                local idx = 1;

                for i2, a in ipairs( self.db.profile.auctionaddonsorder ) do
                    if( self.vars.auctionaddons[a].stop ) then
                        idx = i2;
                        break;
                    end--if
                    idx=i2 + 1;
                end--for
                table.insert( self.db.profile.auctionaddonsorder, idx, addon); -- Add addon that are loaded
                changed = true;
            end--if
        end--if
    end--for
    return changed;
end--Broker_GarbageFu:InitAuctionAddonOrder()

-------------------------------------------------------------------------------------------
-- Auction menu functions
-------------------------------------------------------------------------------------------

function Broker_GarbageFu:BuildAuctionMenu( level, value, inTooltip, value2 )
    if( level == 1 ) then
        dewdrop:AddLine( "text", L["Use Auction price"], "hasArrow", true, "value", "auction" );
    elseif( level == 2 ) then
        if( value == "auction" ) then
            -- Use Auction price
            dewdrop:AddLine( "text", L["Use Auction price"], "isTitle", true );
            dewdrop:AddLine( "text", L["Auction Threshold"], "hasArrow", true, "value", "auctionthreshold" );
            dewdrop:AddLine( "text", L["Auction Sets"], "hasArrow", true, "value", "auctionsets" );
            dewdrop:AddLine( "text", L["Auction Items"], "hasArrow", true, "value", "auctionitem" );
        end--if
    elseif( level == 3 ) then
        if( value2 == "auction" ) then
            if ( value == "auctionthreshold" ) then
                dewdrop:AddLine( "text", L["Auction Threshold"], "isTitle", true );
                for a = 1, 7 do
                    dewdrop:AddLine( "text", self.vars.colors[a].hex .. self.vars.colors[a].desc .. "|r",
                        "checked", a == self:GetAuctionThreshold(),
                        "arg1", self, "arg2", a, "isRadio", true,
                        "func", "SetAuctionThreshold" );
                end--for
                dewdrop:AddLine( "text", L["None"],
                    "checked", 8 == self:GetAuctionThreshold(),
                    "arg1", self, "arg2", 8, "isRadio", true,
                    "func", "SetAuctionThreshold" );
            elseif ( value == "auctionsets" ) then
                -- Auction - Sets
                dewdrop:AddLine( "text", L["Auction Sets"], "isTitle", true );
                for n, i in ipairs( self.vars.sets ) do
                    local checked = self:IsAuctionSetSelected( n );
                    local checkIcon = "Interface\\Buttons\\UI-CheckBox-Check";

                    if( self:IsChildAuctionSetSelected( n ) ) then
                        checked = true;
                        checkIcon = "Interface\\Buttons\\UI-CheckBox-Check-Disabled";
                    end--if
                    dewdrop:AddLine( "text", i.name, 
                        "checked", checked, "checkIcon", checkIcon,
                        "hasArrow", i.sub ~= nil, "value", n,
                        "arg1", self, "arg2", n, "func", "ToggleAuctionSet" );
                end--for
            elseif ( value == "auctionitem" ) then
                -- Auction - Items
                dewdrop:AddLine( "text", L["Auction Items"], "isTitle", true );

                local tbl = self:GetAuctionItemTable();

                for _, i in ipairs( tbl ) do
                    dewdrop:AddLine( "text", self.vars.colors[i.qual].hex .. i.name, 
                        "checked", self:IsAuctionItem( i.id ),
                        "arg1", self, "arg2", i.id, "func", "ToggleAuctionItem");
                end--for
            end--if
        end--if
    elseif( level == 4 ) then
        if( value2 == "auctionsets" ) then
            dewdrop:AddLine( "text", L["Auction Set"] .. " " .. self.vars.sets[value].name, "isTitle", true );
            for n, i in ipairs( self.vars.sets[value].sub ) do
                dewdrop:AddLine( "text", i.name, 
                    "checked", self:IsAuctionSetSelected( value, n ),
                    "arg1", self, "arg2", value, "arg3", n, "func", "ToggleAuctionSet" );
            end--for
        end--if
    end--if
end--Broker_GarbageFu:BuildAuctionMenu( level, value, inTooltip, value2 )

function Broker_GarbageFu:BuildAuctionOptionMenu( level, value, inTooltip, value2 )
    if( level == 1 ) then
        dewdrop:AddLine( "text", L["Auction addon order"], "hasArrow", true, "value", "auctionorder" );
        dewdrop:AddLine( "text", L["Cache Auctioneer"], "checked", self:IsCachedAuctioneerUsed(),
            "arg1", self, "func", "CacheAuctioneer",
            "tooltipTitle", L["Cache Auctioneer"], "tooltipText", L["CacheAuctioneerDesc"] );
        if( self:IsCachedAuctioneerUsed() ) then
            dewdrop:AddLine( "text", L["Clear Auctioneer cache"], 
                "arg1", self, "func", "ClearAuctioneerCache" );
        end--if
    elseif( level == 2 ) then
        if( value == "auctionorder" ) then
            dewdrop:AddLine( "text", L["Auction addon order"], "isTitle", true );
            for i, a in ipairs( self.db.profile.auctionaddonsorder ) do
                dewdrop:AddLine( "text", tostring( i ) .. ". " .. self.vars.auctionaddons[a].name,
                "arg1", self, "arg2", i, "func", "ReorderAuctionAddon" );
            end--for
        end--if
    end--if
end--Broker_GarbageFu:BuildAuctionOptionMenu( level, value, inTooltip, value2 )

