--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

-- $Revision: 2 $
-- $Date: 2008-12-05 16:29:04 +0000 (Fri, 05 Dec 2008) $
if( not Broker_GarbageFu ) then
    return;
end--if
------------------------------------------------------------------------------------------------------
-- Vendor Price functions
------------------------------------------------------------------------------------------------------
local L = LibStub( "AceLocale-3.0" ):GetLocale( "Broker_GarbageFu" );
local dewdrop = LibStub( "Dewdrop-2.0" );

 Broker_GarbageFu.vars.vendoraddons = {
    ["itemprice"] = { name = "GetSellValue", func = "GetItemPrice", check = "GetSellValue"},
    ["sellfish"] = { name = "SellFish", func = "GetSellFishVendorPrice", check = "SellFishDB" },
    ["informant"] = { name = "Informant", func = "GetInformantVendorPrice", check = "Informant" },
    ["kc_items"] = { name = "KC_Items", func = "GetKCIVendorPrice", check = "KC_ItemsDB" },
    ["wowecon"] = { name = "WoWEcon", func = "GetWoWEconVendorPrice", check = "WOWEcon_Enabled" },
    ["itemsync"] = { name = "ItemSync", func = "GetItemSyncVendorPrice", check = "ItemSync" },
    ["pricemaster"] = { name = "PriceMaster", func = "GetPriceMasterVendorPrice", check = "CompletePrices" },
    ["ludwig_sellvalue"] = { name = "Ludwig_SellValue", func = "GetLudwigSellValueVendorPrice", check = "Ludwig_SellValues" },
    ["colalight"] = { name = "ColaLight", func = "GetColaLightVendorPrice", check = "ColaLight" },
    ["lootlink"] = { name = "LootLink", func = "GetLootLinkVendorPrice", check = "LootLink" },
    ["-GFP-"] = { name = "GarbageFu_Price", func = "GetLocalVendorPrice", check = "GarbageFu_Prices" },
    ["-DNU-"] = { name = L["-- Don't use the addons below --"], func = "", check = "Broker_GarbageFu", stop = true },
};
Broker_GarbageFu.vars.vendoraddonsorder = {
    "itemprice",
    "sellfish",
    "informant",
    "kc_items",
    "wowecon",
    "itemsync",
    "pricemaster",
    "ludwig_sellvalue",
    "colalight",
    "-GFP-",
    "-DNU-",
};

function Broker_GarbageFu:InitVendor()
    if self:IsUsingGarbagePrices() then
        self:LoadGarbagePrices();
    end--if
    self:InitVendorAddonOrder();
end--Broker_GarbageFu:InitVendor()

-- Get Vendor price
function Broker_GarbageFu:GetVendorValue( item )
    if( item.id and item.qual ) then
        item.value, item.pricetype = self:GetUserOverridePrice( item );

        for _, addon in ipairs( self.db.profile.vendoraddonsorder ) do
            if( item.value ) then
               break;
            end--if
            if( self.vars.vendoraddons[addon] ) then
                if( self.vars.vendoraddons[addon].stop ) then
                    break;
                end--if
                item.value, item.pricetype = self[self.vars.vendoraddons[addon].func]( self, item );
            end--if
        end--for
    end--if

    if( item.value ) then
        item.totvalue = item.value * item.stack;
    else
        item.value = 0; -- No item value know
        item.totvalue = 0; -- No item value know
    end--if
end--Broker_GarbageFu:GetVendorValue( item )

-- Price Types. First char: U = User, V = Vendor, A = Auction
--              Second char: Addon the price was gotten from.
--                        G = GetSellValue, K = KC_Items, W = WoWEcon, I = Informant, P = PriceMaster, L = Ludwig, T = Local table

-- Get item price from the User Override table
function Broker_GarbageFu:GetUserOverridePrice( item )
    if self.db.global.overrideprices[item.id] then
        return self.db.global.overrideprices[item.id], "|cffff7700UO|r";
    end--if
end--Broker_GarbageFu:GetUserOverridePrice( item )

-- ItemPrice
function Broker_GarbageFu:GetItemPrice( item )
    if( GetSellValue ) then
        return GetSellValue(item.id), "|cff0077ffVG|r";
    end--if
end--Broker_GarbageFu:GetItemPrice( item )

-- SellFish
function Broker_GarbageFu:GetSellFishVendorPrice( item )
    if( SellFishDB ) then
        return GetSellValue( item.id ), "|cff0077ffVS|r";
    end--if
end--Broker_GarbageFu:GetSellFishVendorPrice( item )

-- KC_Items
function Broker_GarbageFu:GetKCIVendorPrice( item )
    if( KC_ItemsDB ) then
        local kv, vs;
    
        kv = KC_ItemsDB[item.id];
        if( kv ) then
            _, _, vs = kv:find( ",(%d+),%d+" );
        end--if
        if( vs and ( vs ~= "0" ) ) then
            return tonumber( vs ), "|cff0077ffVK|r";
        end--if
    end--if
end--Broker_GarbageFu:GetKCIVendorPrice( item )

-- Auctioneer/Informant
function Broker_GarbageFu:GetInformantVendorPrice( item )
    if( Informant ) then
        local itemData = Informant.GetItem( item.id );

        if( itemData and itemData["buy"] and ( itemData["buy"] ~= 0 ) ) then
            return itemData["sell"], "|cff0077ffVI";
        end--if
    end--if
end--Broker_GarbageFu:GetInformantVendorPrice( item )

-- WoWEcon
function Broker_GarbageFu:GetWoWEconVendorPrice( item )
    if( WOWEcon_Enabled ) then
        local price = WOWEcon_GetVendorPrice_ByLink( item.link );

        if( price and ( price > 0 ) ) then
            return price, "|cff0077ffVW|r";
        end--if
    end--if
end--Broker_GarbageFu:GetWoWEconVendorPrice( item )

-- PriceMaster doesn't have a price asking function, so we have to tap directly in to it's data. Ugly!
function Broker_GarbageFu:GetPriceMasterVendorPrice( item )
    if( CompletePrices and CompletePrices[item.id] and CompletePrices[item.id].p ) then
        return CompletePrices[item.id].p, "|cff0077ffVP|r";
    end--if
end--Broker_GarbageFu:GetPriceMasterVendorPrice( item )

-- Ludwig_SellValue
function Broker_GarbageFu:GetLudwigSellValueVendorPrice( item )
    local items = tostring( item.id );

    if( Ludwig_SellValues and Ludwig_SellValues[items] ) then
        return Ludwig_SellValues[items], "|cff0077ffVL|r";
    end--if
end--Broker_GarbageFu:GetLudwigSellValueVendorPrice( item )

-- ItemSync
function Broker_GarbageFu:GetItemSyncVendorPrice( item )
    if( ItemSync ) then
        local r = { ItemSync:_split( ItemSync.db.global[ItemSync.realm]["items"][item.id], "°" ) };

        if( r and r[2] ) then
            local price = tonumber( r[2] );

            return price, "|cff0077ffVIS|r";
        end--if
    end--if
end--Broker_GarbageFu:GetItemSyncVendorPrice( item )

-- ColaLight
function Broker_GarbageFu:GetColaLightVendorPrice( item )
    local items = tostring( item.id )

    if( ColaLight and ColaLight.db.global.SellValues[items] ) then
        return ColaLight.db.global.SellValues[items], "|cff0077ffVC|r";
    end--if
end--Broker_GarbageFu:GetColaLightVendorPrice( item )

-- LootLink
function Broker_GarbageFu:GetLootLinkVendorPrice( item )
    if( LootLink ) then
        local linkTable = LootLink.Database[item.id];

        if( linkTable and linkTable[6] ) then
            return linkTable[6], "|cff0077ffVLL|r";
        end--if
    end--if
end--Broker_GarbageFu:GetLootLinkVendorPrice( item )

-- Look it up in our own price table
function Broker_GarbageFu:GetLocalVendorPrice( item )
    if( self.vars.vendorprices ) then
        if( self.vars.pricecache[item.id] ) then
            return self.vars.pricecache[item.id], "|cff0077ffVT".."|r";
        end--if

        local _, _, price = self.vars.vendorprices:find( " " .. item.id .. ":(%d+)" );

        if price then
            self.vars.pricecache[item.id] = tonumber( price )
            return self.vars.pricecache[item.id], "|cff0077ffVT|r";
        end--if
    end--if
end--Broker_GarbageFu:GetLocalVendorPrice( item )

-------------------------------------------------------------------------------------------
-- GarbageFu_Prices functions
-------------------------------------------------------------------------------------------

-- Load our own price table.
GarbageFu_Prices = false;
function Broker_GarbageFu:LoadGarbagePrices( db )
    if( not IsAddOnLoaded( "GarbageFu_Prices" ) ) then
        LoadAddOn( "GarbageFu_Prices" );
    end--if
    if( GarbageFu_PriceDb ) then
        self.vars.vendorprices = GarbageFu_PriceDb;
        GarbageFu_Prices = true;
        return true;
    elseif( GarbageFu_PriceTbl ) then
        self.vars.vendorprices = "";
        self.vars.pricecache = GarbageFu_PriceTbl;
        GarbageFu_Prices = true;
        return true;
    else
         self:MyPrint( L["Can't load Broker_GarbageFu_Price. Check that it exists and are enabled"] );
         return;
    end--if
end--Broker_GarbageFu:LoadGarbagePrices( db )

function Broker_GarbageFu:IsGarbagePricesAvailable()
    return not select( 6, GetAddOnInfo( "GarbageFu_Prices" ) );
end

-------------------------------------------------------------------------------------------
-- Drop down menu functions
-------------------------------------------------------------------------------------------

function Broker_GarbageFu:IsUsingGarbagePrices()
    return self.db.profile.garbageprices;
end--Broker_GarbageFu:IsUsingGarbagePrices()

function Broker_GarbageFu:ToggleUseGarbagePrices()
    if( not self.db.profile.garbageprices ) then
        self.db.profile.garbageprices = self:LoadGarbagePrices();
        if( self.db.profile.garbageprices ) then
            self:InitVendorAddonOrder()
            self:UpdateAllItemValues();
        end--if
    else
        self.db.profile.garbageprices = false;
        GarbageFu_Prices = false;
        self.vars.vendorprices = nil;
        self.vars.pricecache = {};
        self:InitVendorAddonOrder()
        self:UpdateAllItemValues();
    end--if
end--Broker_GarbageFu:ToggleUseGarbagePrices()

function Broker_GarbageFu:ReorderVendorAddon( index )
    if( index == 1 ) then -- move down
        table.insert( self.db.profile.vendoraddonsorder, index + 1, table.remove( self.db.profile.vendoraddonsorder, index ) );
    else -- move up
        table.insert( self.db.profile.vendoraddonsorder, index - 1, table.remove( self.db.profile.vendoraddonsorder, index ));
    end--if
    self:UpdateAllItemValues();
end--Broker_GarbageFu:ReorderVendorAddon( index )

-------------------------------------------------------------------------------------------
-- Vendor init functions
-------------------------------------------------------------------------------------------

function Broker_GarbageFu:CheckVendorAddonLoaded( addon )
    addon = addon:lower();
    if( self.vars.vendoraddons[addon] and self:InitVendorAddonOrder() ) then
        self:UpdateAllItemValues();
    end--if
end--Broker_GarbageFu:CheckVendorAddonLoaded( addon )

-------------------------------------------------------------------------------------------
-- Vendor init functions
-------------------------------------------------------------------------------------------

function Broker_GarbageFu:InitVendorAddonOrder()
    local changed = false;

    for _, addon in pairs( self.vars.vendoraddonsorder ) do
        local found = false;

        for i, order in ipairs( self.db.profile.vendoraddonsorder ) do
            if( order == addon ) then
                if( not _G[self.vars.vendoraddons[addon].check] ) then
                    table.remove( self.db.profile.vendoraddonsorder, i ); -- Remove addons not loaded
                    changed = true;
                end--if
                found = true;
                break;
            end--if
        end--for
        if( not found ) then
            if( _G[self.vars.vendoraddons[addon].check] ) then
                local idx=1;

                for i2, a in ipairs( self.db.profile.vendoraddonsorder ) do
                    if( self.vars.vendoraddons[a] and self.vars.vendoraddons[a].stop ) then
                        idx = i2;
                        break;
                    end--if
                    idx=i2 + 1;
                end--for
                table.insert( self.db.profile.vendoraddonsorder, idx, addon );  -- Add addon that are loaded
                changed = true;
            end--if
        end--if
    end--for
    return changed;
end--Broker_GarbageFu:InitVendorAddonOrder()

-------------------------------------------------------------------------------------------
-- Vendor menu functions
-------------------------------------------------------------------------------------------

function Broker_GarbageFu:BuildVendorOptionMenu( level, value, inTooltip, value2 )
    if( level == 1 ) then
        dewdrop:AddLine( "text", L["Use GarbageFu Prices"], "checked", self:IsUsingGarbagePrices(),
            "arg1", self, "func", "ToggleUseGarbagePrices",
            "disabled", not ( self:IsGarbagePricesAvailable() or self:IsUsingGarbagePrices() ),    -- keep enabled if we're configged to use so we can bloody turn it off!
            "tooltipTitle", L["Use GarbageFu Prices"], "tooltipText", L["GarbageFu_Prices TooltipText"] );
        dewdrop:AddLine( "text", L["Vendor price addon order"], "hasArrow", true, "value", "vendororder" );
    elseif( ( level == 2 ) and ( value == "vendororder" ) ) then
        dewdrop:AddLine( "text", L["Vendor price addon order"], "isTitle", true );

        for i, a in ipairs( self.db.profile.vendoraddonsorder ) do
            if( not self.vars.vendoraddons[a] ) then
                table.remove( self.db.profile.vendoraddonsorder, i );
            else
                dewdrop:AddLine( "text", tostring(i)..". "..self.vars.vendoraddons[a].name, "arg1", self, "arg2", i, "func", "ReorderVendorAddon" );
            end--if
        end--for
    end--if
end--Broker_GarbageFu:BuildVendorOptionMenu( level, value, inTooltip, value2 )
